/*
    Fichier trtligne.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence.fr

    Traductions des droits d'utilisation dans les fichiers :
    licence.de , licence.en , licence.es , licence.it
    licence.nl , licence.pt , licence.eo , licence.eo-utf


    Bibliothque de fonctions permettant de :

    - mmoriser le mode d'encodage et le jeu de caractres du message
    - convertir les accents et caractres spciaux cods sur 7 bits
      ou/et utilisant un jeu de caractres non ISO-8859 dans le message
    - rcuprer un nom de fichier joint
*/

#include <stdio.h>
#include "buflect.h"
#include "encodage.h"
#include "trtligne.h"


/* prototypes de fonctions locales  ce source */

void sup_quoted ();
void conv_isomac ();
void conv_utf8 ();
void conv_carhtm ();


/* variable globale au source */

static int charset_texte;      // jeu de caractres accentus du texte



/* retourne la valeur du champ Content-Type */

int recup_ctype ()
{
    int i;   // numro de caractre dans buf_lect


    // se positionner aprs le nom de champ
    i = 13;

    // sauter les blancs
    while (buf_lect [i] == ' ')
        i++;

    // dtection du type de contenu
    if (tolower (buf_lect [i]) == 't')
    {
        // discrimination entre text/plain et text/html
        if (tolower (buf_lect [i + 5]) == 'p')
            return (TextPlain);
        else if (tolower (buf_lect [i + 5]) == 'h')
            return (TextHtml);
        else
            return (AutreType);
    }
    else if (tolower (buf_lect [i]) == 'm')
    {
        // discrimination entre les diffrents mode multipart
        if (tolower (buf_lect [i + 10]) == 'a')
            // multipart/alternative
            return (MultipAlter);

        else if (tolower (buf_lect [i + 10]) == 'm')
            // multipart/mixed
            return (MultipMixed);

        else if (tolower (buf_lect [i + 10]) == 'r')
        {
            if (tolower (buf_lect [i + 12]) == 'l')
                // le mode multipart/related est trait comme multipart/mixed
                return (MultipMixed);
            else if (tolower (buf_lect [i + 12]) == 'p')
                // multipart/report
                return (MultipRep);
        }
        else
            return (Multipart);
    }
    else
        return (AutreType);
}



/* mmorise dans  encodage_texte  la valeur
   du champ Content-Transfer-Encoding */

int mem_encodage ()
{
    int i;   // numro de caractre dans buf_lect


    // se positionner aprs le nom de champ
    i = 26;

    // sauter les blancs
    while (buf_lect [i] == ' ')
        i++;

    // dtection de l'encodage (simplifie, on analyse un caractre)
    if (tolower (buf_lect [i]) == 'q')
        encodage_texte = QuotedPrint;
    else if (tolower (buf_lect [i]) == 'b')
        encodage_texte = Base64;
    else
        encodage_texte = xBits;

    // seules certaines applications utilisent la valeur de retour
    return (encodage_texte);
}



/* mmorise dans  charset_texte  le jeu de
   caractres utilis dans le texte du mail
   retourne 1 si champ charset trouv dans la ligne, 0 sinon */

int mem_charset ()
{
    int i;  // compteur


    // initialisation
    i = 1;

    // chercher le mot  charset  dans la ligne et sortir s'il n'y est pas
    while (tolower (buf_lect [i]) != 'c' || tolower (buf_lect [i+1]) != 'h')
        if (! buf_lect [++i])
            return (0);

    // se positionner aprs le caractre  =
    while (buf_lect [i++] != '=')
        if (! buf_lect [i])
            return (0);

    // sauter galement le  "  s'il y est
    if (buf_lect [i] == '"')
        i++;

    // dtection du charset (simplifie, on analyse un caractre)
    if (tolower (buf_lect [i]) == 'i')
    {
        // jeu de caractre ISO
        charset_texte = Iso8859;

        // vrifier si variante Mac
        while (tolower (buf_lect [i]) != 'x' || buf_lect [i+1] != '-'
                                    || tolower (buf_lect [i+2]) != 'm')
            if (! buf_lect [++i])
                return (1);

        // variante Mac trouve
        charset_texte = IsoMac;
    }
    else if (tolower (buf_lect [i]) == 'u')
        charset_texte = Utf8;
    else
        charset_texte = xCharset;

    // on a trouv le charset
    return (1);
}



/* convertit la ligne lue en interprtant les caractres spciaux */

void majligne ()
{
    int i;  // compteur


    // suppression du . ventuel en premire colonne
    if (buf_lect [0] == '.')
    {
        i = 0;

        do
            buf_lect [i] = buf_lect [i + 1];
        while (buf_lect [i++] != '\0');
    }

    // suppression de l'encodage quoted-printable
    if (encodage_texte == QuotedPrint)
        sup_quoted ();

    // autre encodage, on rajoute juste un passage  la ligne
    else
    {
        i = strlen (buf_lect);
        buf_lect [i++] = '\n';

        // terminaison de la chaine de caractres
        buf_lect [i] = '\0';
    }

    // conversion des jeux de caractres non ISO

    if (charset_texte == Utf8)
        // conversion du jeu de caractres UTF8
        conv_utf8 ();

    // Normalement, le besoin de conversion des caractres Mac devrait
    // tre signale dans le charset.
    // Toutefois, il peut tre prfrable de la faire systmatiquement
    else // if (charset_texte == IsoMac)
        // traitement des caractres spciaux sur Mac
        conv_isomac ();

    // convertir les caractres HTML, par exemple les &#8217; en apostrophes
    // ce genre de squence NE DEVRAIT JAMAIS se trouver dans un TEXTE de
    // mail correctement gnr (mais on en trouve !!!!)
    conv_carhtm ();
}



/* suppression de l'encodage quoted-printable */

void sup_quoted ()
{
    int carcode;  // caractre correspondant  un codage quoted-printable
    int i, j;     // compteurs


    // initialisation
    i = 0;
    j = 0;

    // exploration de toute la ligne
    while (buf_lect [i] != '\0')
    {
        // si caractre quoted printable  convertir
        if (buf_lect [i] == '=' &&
             ((buf_lect [i+1] >= '0' && buf_lect [i+1] <= '9') ||
              (buf_lect [i+1] >= 'A' && buf_lect [i+1] <= 'F')))
        {
            // on dcode le caractre correspondant
            carcode = hexa (buf_lect + ++i);

            // si ce n'est pas un caractre de controle ou si c'est un LF
            if (carcode > 0x1F || carcode == '\n')
                // le mmoriser
                // Remarque : la conversion d'un LF peut tre une source de
                //            problme d'affichage notamment avec vmailfic
                buf_lect [j++] = carcode;

            // sinon, si caractre de controle autre que CR
            else if (carcode != '\r')
                // mmoriser un blanc pour limiter les problmes d'affichage
                buf_lect [j++] = ' ';

            // se positionner sur le caractre suivant
            i = i + 2;
        }
        // sinon on conserve le caractre courant et on passe au suivant
        else
            buf_lect [j++] = buf_lect [i++];
    }

    // traitement de la fusion ventuelle de 2 lignes
    if (buf_lect [i - 1] == '=')
        j--;
    else
        buf_lect [j++] = '\n';

    // terminaison de la chaine de caractres
    buf_lect [j] = '\0';
}



/* conversion des caractres spciaux sur Mac */

void conv_isomac ()
{
    int i;   // compteur


    for (i = 0; buf_lect [i] != '\0'; i++)
    {
        switch (buf_lect [i])
        {
            case 0x92 : buf_lect [i] = '\'';
                        break;
            case 0x93 : buf_lect [i] = '';
                        break;
            case 0x94 : buf_lect [i] = '';
                        break;
            case 0x9C : buf_lect [i] = '';
            default   : break;
        }
    }
}



/* conversion du jeu de caractres UTF8 */

void conv_utf8 ()
{
    /* indique que le caractre qui suit le  devra tre converti
       on utilise une variable statique pour garder l'info en mmoire dans le
       cas o le  et le caractre qui suit seraient sur 2 lignes distinctes */
    static int decalsuiv = 0;

    int i, j;   // compteurs


    // initialisation
    i = 0;
    j = 0;

    do
    {
        switch (buf_lect [i])
        {
            // si caractre  , on convertira le caractre qui le suit
            case 0xC3 : decalsuiv = 1;
            case 0xC2 : break; // on saute le caractre  ou 

            // fin de buffer trait sans remettree  jour decalsuiv
            case '\0' : buf_lect [j++] = '\0';
                        break;

            // pour tous les autres caractres, 2 cas possibles
            default   : if (decalsuiv)
                        {
                            // conversion du caractre qui suit le 
                            buf_lect [j++] = buf_lect [i] | 0x40;
                            decalsuiv = 0;
                        }
                        else
                            // ou simple copie du caractre courant
                            buf_lect [j++] = buf_lect [i];
        }
    }
    while (buf_lect [i++] != 0);
}



/* convertir les caractres HTML, par exemple les &#8217; en apostrophes */

void conv_carhtm ()
{
    int valeur;    // valeur numrique  convertir en caractre
    int i, j, k;   // compteurs


    // initialisation
    i = 0;
    j = 0;

    do
    {
        // si on trouve les caractres &#  la suite
        if (buf_lect [i] == '&' && buf_lect [i+1] == '#')
        {
            // initialisation
            valeur = 0;
            k = 2;

            // rcuprer la valeur numrique qui suit
            while ('0' <= buf_lect [i+k] && buf_lect [i+k] <= '9')
                valeur = (valeur * 10) + (buf_lect [i+k++] - '0');

            // si on a trouve une valeur numrique suivie de ;
            if (k > 2 && buf_lect [i+k] == ';')
            {
                // convertir le caractre
                switch (valeur)
                {
                    // &#339; est remplac par 
                    case 339 : buf_lect [j++] = '';
                               break;

                    // &#8211; est remplac par un tiret
                    case 8211: buf_lect [j++] = '-';
                               break;

                    // &#8216; &#8217; sont remplacs par une apostrophe
                    case 8216:
                    case 8217: buf_lect [j++] = '\'';
                               break;

                    // caractre non encore converti, il sera recopi
                    default  : buf_lect [j++] = '&';
                               buf_lect [j++] = '#';
                               k = 1;
                }

                // et avancer dans le tableau
                i = i + k;
            }
            // sinon
            else
                // simple copie du caractre courant
                buf_lect [j++] = buf_lect [i];
        }
        // sinon
        else
            // simple copie du caractre courant
            buf_lect [j++] = buf_lect [i];
    }
    // on fait ce traitement jusqu' la fin de la ligne
    while (buf_lect [i++] != 0);
}



/* retourne la position d'un nom de fichier joint dans la ligne courante */

int posnomfic ()
{
    char chaineref [] = "name="; // chaine  chercher pour trouver fichier joint
    int i, j;                    // compteurs


    if (strlen (buf_lect) < 7)
        // la ligne est trop courte pour contenir un nom de fichier joint
        return (0);

    // le mot cl name ou filename ne commence pas en premire colonne
    i = 1;

    // recherche de la chaine name =
    do
    {
        // test sur le premier caractre
        if (tolower (buf_lect [i]) == *chaineref)
        {
            // et les suivants
            j = 1;

            while (tolower (buf_lect [i+j]) == chaineref [j])
                j++;

            // trouv ?
            if (chaineref [j] == '\0')
            {
                // sauter le " ventuel
                if (buf_lect [i+j] == '"')
                    i++;

                // et retourner la position du nom
                return (i + j);
            }
        }
    }
    // continuer la recherche
    while (buf_lect [++i] != '\0');

    // fichier joint non trouv
    return (0);
}
