/*
    Fichier trtsection.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence.fr

    Traductions des droits d'utilisation dans les fichiers :
    licence.de , licence.en , licence.es , licence.it
    licence.nl , licence.pt , licence.eo , licence.eo-utf


    Bibliothque de fonctions permettant de parcourir les diffrentes
    sections d'un mail en mode multipart, pour accder  la zone texte
    et rcuprer la liste des fichiers joints.
*/

#include <stdio.h>
#include "buflect.h"
#include "fmail.h"
#include "trtligne.h"
#include "encodage.h"
#include "trtsection.h"
#include "trtbordure.h"



/* rcuprer la valeur du content-type, et le charset ou la bordure */

void typeorig (long pos_ctype)
{
    // initialisation
    nbordures = 0;

    // si un champ Content-Type figure dans l'entte du mail
    if (pos_ctype >= 0)
    {
        // se positionner sur ce champ
        fseek (fmail, pos_ctype, SEEK_SET);
        lire_fmail ();

        // rcuprer la valeur du champ
        ctypeorig = recup_ctype ();

        // si son type correspond au texte du mail
        if (ctypeorig == TextPlain)
        {
            // mmoriser le jeu de caractres
            while (*buf_lect && ! mem_charset ())
                lire_fmail ();
        }
        // sinon si mode multipart
        else if (ctypeorig & Multipart)
        {
            // mmoriser la bordure
            while (*buf_lect && ! mem_boundary ())
                lire_fmail ();
        }
    }
    else
        // si pas de champ Content-Type, on affichera quand mme le mail
        ctypeorig = TextPlain;

    // pour l'instant, le type principal du mail est le type courant
    ctype = ctypeorig;
}



/* avancer jusqu' la section suivante si
   on n'est pas sur un marqueur de section */

void prochaine_section ()
{
    // tant que bordure non trouve et non fin de mail
    // remarque : il est important de tester la bordure
    //            avant de passer  la ligne
    while (! surbordure () && lire_fmail ())
        ;  // chercher la bordure sur la ligne suivante

    // si bordure trouve, lire la ligne suivante
    if (buf_lect [0] == '-')
        lire_fmail ();
}



/* mmorise le sous type de la premire section imbriqu
   dans un mail multipart/mixed et sa bordure s'il y a lieu */

void mem_soustype ()
{
    // avancer jusqu' la prochaine section
    prochaine_section ();

    // mmoriser ses caractristiques
    do
    {
        // content-type
        if (start ("Content-Type"))
        {
            // mmoriser le type de la section
            ctype = recup_ctype ();

            // si on a trouv un autre mode multipart imbriqu
            if (ctype & Multipart)
            {
                // mmoriser sa bordure
                while (*buf_lect && ! mem_boundary ())
                    lire_fmail ();
            }
        }

        // mode d'encodage
        else if (start ("Content-Transfer-Encoding"))
            mem_encodage ();

        // passer  la ligne suivante
        lire_fmail ();
    }
    // on s'arrtera lorsqu'on aura trouv un nouveau type
    // ou en fin d'entte de section, (ou de mail)
    while (ctype == MultipMixed && buf_lect [0] != '\0');
}



// se positionner sur la section texte du mail (si l'on n'y est pas)

void posit_texte ()
{
    posit_section (TextPlain);
}



// se positionner sur la section text/html du mail (si l'on n'y est pas)

void posit_texthtm ()
{
    posit_section (TextHtml);
}



// se positionner (si l'on n'y est pas) sur la section text/plain
// ou text/html du mail, en fonction du paramtre  typetexte

void posit_section (int typetexte)
{
    int dansmail = 1;    // indicateur de fin de mail
    int OK_charset = 0;  // indicateur : charset trouv


    // tant qu'on n'a pas trouv la section contenant le texte du mail
    while (ctype != typetexte && dansmail)
    {
        // si une bordure a t mmorise
        if (nbordures)
            // se positionner sur la section suivante
            prochaine_section ();

        // mmoriser ses caractristiques
        do
        {
            // content-type et charset ventuel
            if (start ("Content-Type"))
            {
                ctype = recup_ctype ();
                OK_charset = mem_charset ();
            }

            // mode d'encodage
            else if (start ("Content-Transfer-Encoding"))
                mem_encodage ();

            // avancer d'une ligne
            dansmail = lire_fmail ();
        }
        // jusqu' type trouv ou fin d'entte
        while (ctype != typetexte && buf_lect [0] != '\0');
    }

    // on est sur la section contenant le texte du mail
    // on va mmoriser le charset et le mode d'encodage s'il y a lieu
    while (buf_lect [0] != '\0')
    {
        if (start ("Content-Transfer-Encoding"))
            mem_encodage ();

        else if (! OK_charset)
            OK_charset = mem_charset ();

        // passer  la ligne suivante
        lire_fmail ();
    }
}



// lister les pices jointes

void liste_pj ()
{
    int  posjoint;        // position du nom du fichier joint
    char nomjoint [80];   // nom du fichier joint
    int  nbext;           // nombre d'extentions du fichier
    int  i;               // compteur


    // on ne conserve que la bordure de premier niveau
    // (permet de sauter le texte HTML en mode multipart/alternative)
    nbordures = 1;

    // rpter
    do
    {
        // se positionner sur la prochaine section si on n'y est pas dj
        prochaine_section ();

        // chercher et mmoriser le nom du fichier joint s'il y en a un
        do
        {
            majligne ();
            posjoint = posnomfic ();

            // si trouv
            if (posjoint)
            {
                // rcuprer son nom
                strcpy (nomjoint, "-> Fichier joint : ");
                i = strlen (nomjoint);
                nbext = 0;

                // recopie du nom de fichier
                while (buf_lect [posjoint] != '"' && buf_lect [posjoint])
                {
                    // sans les (horribles) blancs qu'il pourrait contenir !!!
                    if (buf_lect [posjoint] != ' ')
                        nomjoint [i++] = buf_lect [posjoint];

                    // passer au caractre suivant
                    posjoint++;

                    // on compte les . (nombre d'extentions) du fichier
                    //  partir du 2me caractre du nom
                    if (buf_lect [posjoint] == '.')
                        nbext ++;
                }

                // terminer la chaine gnre
                nomjoint [i]   = '\n';
                nomjoint [i+1] = '\0';
            }
        }
        while (! posjoint && lire_fmail () && buf_lect [0] != '\0');

        // si fichier joint trouv
        if (posjoint)
        {
            // et que son nom comporte plusieurs extentions
            // if (nbext > 1)  *** test devenu insuffisant en 2004 ***

            // actuellement tous les fichiers joints avec extention peuvent
            // contenir des virus, on accepte quand mme les .exe
            if (nbext > 1 || (nbext == 1 && strcmp (nomjoint + i - 4, ".exe\n")
                             != 0 && strcmp (nomjoint + i - 4, ".EXE\n") != 0))
            {
                // se positionner sur la premire ligne du contenu du fichier
                while (buf_lect [0] != '\0')
                    lire_fmail ();

                lire_fmail ();

                // si le fichier commence par les caractres MZ
                // (on teste l'encodage base64 correspondant)
                if (buf_lect [0] == 'T' && buf_lect [1] == 'V'
                 && buf_lect [2] >= 'o' && buf_lect [2] <= 'r')
                    // il peut s'agir d'un virus MS-DOS / Windows
                    strcpy (nomjoint + i, "     *** VIRUS ? ***\n");
            }

            // mmoriser le fichier joint trouv
            genligne (nomjoint);
        }
    }
    // jusqu'en fin de mail
    while (lire_fmail ());
}



/* affiche un avertissement si le texte est encod base 64 */

void test_encode ()
{
    if (encodage_texte == Base64)
    {
        genligne ("\n");
        genligne ("********* Texte non standard : encodage Base64 *********\n");
    }
}
